<?php

namespace Drupal\node_parts;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\path_alias\AliasManagerInterface;

/**
 * The alias generator class.
 */
final class Alias implements AliasInterface {

  /**
   * Constructs an Alias object.
   */
  public function __construct(
    private readonly AliasManagerInterface $pathAliasManager,
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly AnalyzerInterface $analyzer,
  ) {}

  /**
   * {@inheritDoc}
   */
  public function createUpdateAlias(EntityInterface $entity): void {
    // Get the fields that use paragraph.
    $fields = $this->analyzer->getFields($entity);
    $node_parts = 0;
    foreach ($fields as $field) {
      // Count all node_part paragraphs.
      $references = $entity->get($field)->referencedEntities();
      foreach ($references as $reference) {
        if ($reference->bundle() === 'node_part_paragraph') {
          $node_parts++;
        }
      }
      if ($node_parts > 0) {
        // We have found at least one node_part paragraph, so we can stop
        break;
      }
    }

    if ($node_parts === 0) {
      // No node_part paragraphs found, so we don't need to do anything.
      return;
    }

    // The node path.
    $path = '/node/' . $entity->id();
    // Get the alias for the node.
    $node_alias = $this->pathAliasManager->getAliasByPath($path);
    // Get the language code for the node.
    $langcode = $entity->language()->getId();

    for ($i = 1; $i <= $node_parts; $i++) {
      // Set the path for node part.
      $path = '/node/' . $entity->id() . '/part/' . $i;
      // Set the alias for node part.
      $alias = $node_alias . '/' . $i;
      /**
       * @var \Drupal\path_alias\PathAliasStorage $storage
       */
      $storage = $this->entityTypeManager->getStorage('path_alias');
      // Check if the alias already exists.
      $alias_exists = $storage->loadByProperties(
        [
          'path' => $path,
          'alias' => $alias,
          'langcode' => $langcode,
        ]
      );
      if ($alias_exists) {
        // The alias already exists, so we don't need to create it again.
        continue;
      }
      // Check if the alias part already exists.
      $alias_exists = $storage->loadByProperties(
        [
          'path' => $path,
          'langcode' => $langcode,
        ]
      );
      if ($alias_exists) {
        // The alias part path already exists we need to delete it.
        $storage->delete($alias_exists);
      }

      // Create the alias.
      $storage->create(
        [
          'path' => $path,
          'alias' => $alias,
          'langcode' => $entity->language()->getId(),
          'status' => 1,
        ]
      )->save();
    }
  }

}
