<?php

namespace Drupal\node_parts;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Entity\EntityInterface;

/**
 * The node analyzer class.
 */
final class Analyzer implements AnalyzerInterface {

  /**
   * Constructs an Analyzer object.
   */
  public function __construct(
    protected CacheBackendInterface $cacheBackend,
  ) {}

  /**
   * {@inheritDoc}
   */
  public function getFields(EntityInterface $entity): array {
    $cid = $entity->getEntityTypeId() . ':' . $entity->bundle();
    if ($cache = $this->cacheBackend->get($cid)) {
      // Data is already cached.
      return $cache->data;
    }

    $fields = [];
    foreach ($entity->getFieldDefinitions() as $field_name => $field_definition) {
      // Check if the field is a field collection or entity reference
      if ($field_definition->getType() === 'entity_reference_revisions') {
        // Check if the target type is a paragraph
        if ($field_definition->getSetting('target_type') === 'paragraph') {
          $fields[] = $field_name;
        }
      }
    }

    // Cache the result.
    $tags = ['config:node.field_schema_data.' . $entity->bundle()];
    $this->cacheBackend
      ->set($cid, $fields, CacheBackendInterface::CACHE_PERMANENT, $tags);

    return $fields;
  }

  /**
   * {@inheritDoc}
   */
  public function pageCount(EntityInterface $entity): int {
    $cid = 'node_parts_page_count:' . $entity->id();
    if ($cache = $this->cacheBackend->get($cid)) {
      // Data is already cached.
      return $cache->data;
    }
    $total = 1;
    $fields = $this->getFields($entity);
    if (empty($fields)) {
      // No fields to process, so we don't need to do anything.
      return $total;
    }

    // Loop through the fields and count the number of paragraphs.
    foreach ($fields as $field_name) {
      foreach ($entity->get($field_name)->referencedEntities() as $paragraph) {
        if ($paragraph->bundle() === 'node_part_paragraph') {
          // Increment the total value for each paragraph found.
          $total++;
        }
      }
    }

    // Cache the result.
    $this->cacheBackend
      ->set($cid, $total, CacheBackendInterface::CACHE_PERMANENT,
        ['node:' . $entity->id()]);

    // Return the total page count.
    return $total;
  }

}
