<?php

namespace Drupal\node_parts;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\node_parts\Event\LinkAlterEvent;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * The preprocess class for node parts paragraphs.
 */
final class Preprocess implements PreprocessInterface {

  use StringTranslationTrait;

  /**
   * Constructs a Preprocess object.
   *
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $eventDispatcher
   *   The event dispatcher service.
   */
  public function __construct(
    private readonly EventDispatcherInterface $eventDispatcher,
    private readonly AnalyzerInterface $analyzer,
  ) {}

  /**
   * {@inheritDoc}
   */
  public function preprocess(array &$variables): void {
    /** @var \Drupal\paragraphs\ParagraphInterface $paragraph */
    $paragraph = $variables['paragraph'] ?? NULL;
    if (!is_a($paragraph, 'Drupal\paragraphs\ParagraphInterface')) {
      // Not a paragraph, so we don't need to do anything.
      return;
    }

    // Check if the paragraph is a node part paragraph.
    $bundles = ['node_part_paragraph', 'node_part_link_paragraph'];
    if (!in_array($paragraph->bundle(), $bundles)) {
      // Not a node part paragraph, so we don't need to do anything.
      return;
    }

    // get the page number.
    $page = $variables['elements']['#page'] ?? NULL;
    if ($page === NULL) {
      return;
    }

    /** @var \Drupal\node\NodeInterface $parent */
    $parent = $paragraph->getParentEntity();
    // Get the total number of parts for the parent node.
    $total = $this->analyzer->pageCount($parent);

    $previous_page = $page - 2;
    if ($previous_page >= 0) {
      $previous_url = $previous_page === 0 ?
        Url::fromRoute('entity.node.canonical', ['node' => $parent->id()]) :
        Url::fromRoute(
          'node_parts.entity', [
            'node' => $parent->id(),
            'page' => $previous_page,
          ]
        );
    }

    $previous = [];
    if (isset($previous_url)) {
      // Add the link to the previous part.
      $previous = [
        '#type' => 'link',
        '#title' => t('View previous part'),
        '#url' => $previous_url,
        '#attributes' => [
          'class' => ['node-part-paragraph-link'],
        ],
      ];
    }

    $next = [];
    if ($page < $total) {
      // Create the url for the next part.
      $url = Url::fromRoute(
        'node_parts.entity', [
          'node' => $paragraph->get('parent_id')->getString(),
          'page' => $page,
        ]
      );

      // Add the link to the next part.
      $next = [
        '#type' => 'link',
        '#title' => t('View next part'),
        '#url' => $url,
        '#attributes' => [
          'class' => ['node-part-paragraph-link'],
        ],
      ];
    }

    // Add the variables to the template.
    $variables['content']['page_link'] = [
      '#theme' => 'node_parts_navigation_bar',
      '#label' => $this->t('Page @page of @total',
        ['@page' => $page, '@total' => $total]),
      '#previous' => $previous,
      '#next' => $next,
    ];

    /** @var \Drupal\node_parts\Event\LinkAlterEvent $event */
    $event = new LinkAlterEvent($variables);
    // Let other modules alter the link.
    $this->eventDispatcher->dispatch($event, LinkAlterEvent::class);

    $variables = $event->getVariables();
  }

}
