<?php

namespace Drupal\node_parts;

use Drupal\Core\Entity\Display\EntityViewDisplayInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\node\NodeInterface;
use Drupal\node_parts\Event\AllowedViewModesEvent;
use Drupal\node_parts\Event\RequiredFieldsEvent;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * The renderer service for node parts.
 */
final class Renderer implements RendererInterface {

  /**
   * The current page number.
   *
   * @var int
   */
  protected int $page = 0;

  /**
   * The entity to render.
   *
   * @var \Drupal\node\NodeInterface
   */
  protected NodeInterface $entity;

  /**
   * The fields to contains paragraphs.
   *
   * @var array
   */
  protected array $fields = [];

  /**
   * Constructs a Renderer object.
   */
  public function __construct(
    private readonly RouteMatchInterface $routeMatch,
    private readonly AnalyzerInterface $analyzer,
    private readonly EventDispatcherInterface $eventDispatcher,
  ) {}

  /**
   * {@inheritDoc}
   */
  public function render(array &$build, EntityInterface $entity, ?EntityViewDisplayInterface $display = NULL, string $view_mode = 'full'): array {
    if (!is_a($entity, 'Drupal\node\NodeInterface')) {
      // The entity is not a content entity, so we don't need to do anything.
      return $build;
    }
    // Get the fields that use paragraph.
    $this->fields = $this->analyzer->getFields($entity);
    if (empty($this->fields)) {
      // No fields to process, so we don't need to do anything.
      return $build;
    }

    // Set allowed view modes.
    $view_modes = ['full'];
    $event = new AllowedViewModesEvent($view_modes);
    $this->eventDispatcher->dispatch($event, AllowedViewModesEvent::class);
    $view_modes = $event->getViewModes();

    // the display mode is not full, we don't need to do anything.
    if (!in_array($view_mode, $view_modes)  ) {
      return $build;
    }

    // Set the entity.
    $this->entity = $entity;

    // Get the current page number from the route match.
    $this->page = $this->routeMatch->getParameter('page') ?? 0;

    $this->renderCurrentPage($build);

    return $build;
  }

  /**
   * Renders the current page.
   *
   * @param array $build
   *   The render array.
   */
  protected function renderCurrentPage(array &$build): void {
    // Get the paragraphs and the field.
    [$paragraph_begin, $paragraph_end, $field, $links] = $this->getParagraphs();
    if ($paragraph_begin === NULL && $paragraph_end === NULL) {
      // No part paragraphs found, so we don't need to do anything.
      return;
    }

    // Get the renderer element keys.
    $children = Element::children($build[$field]);
    // Get the renderer elements.
    $render_elements = array_filter(
      $build[$field],
      fn($key) => in_array($key, $children),
      ARRAY_FILTER_USE_KEY
    );

    // Remove the renderer elements from the build array.
    $build[$field] = array_diff_key($build[$field], array_flip($children));

    // Prepare the part paragraphs.
    if ($paragraph_end !== NULL) {
      // Add page number to the part paragraph.
      $render_elements[$paragraph_end]['#page'] = $this->page + 1;
      foreach ($links as $link) {
        if (($paragraph_begin === NULL || $link > $paragraph_begin)
          && $link < $paragraph_end
        ) {
          // Add page number to the part link paragraph.
          $render_elements[$link]['#page'] = $this->page + 1;
        }
      }
    }
    else {
      // Add page number to the part paragraph.
      $render_elements[$paragraph_begin]['#page'] = $this->page + 1;
    }

    // Add the renderer elements to the build array.
    $visible_elements = $this->filterByKeyRange(
      $render_elements, $paragraph_begin, $paragraph_end
    );
    $build[$field] += array_values($visible_elements);

    // Add the top navigation paragraph of the page.
    $top_paragraph = $paragraph_end !== NULL ?
      $render_elements[$paragraph_end] ?? [] :
      $render_elements[$paragraph_begin] ?? [];
    array_unshift($build[$field], $top_paragraph);

    if ($paragraph_begin === NULL) {
      // This is the node page we render to the first part of the paragraph.
      return;
    }

    if ($paragraph_end === NULL) {
      // Add navigation bar to the last part of the page.
      $build[$field][] = $render_elements[$paragraph_begin] ?? [];
    }

    /** @var \Drupal\paragraphs\ParagraphInterface[] $references */
    $references = $this->entity->get($field)->referencedEntities();
    /** @var \Drupal\paragraphs\ParagraphInterface|null $part */
    $part = $references[$paragraph_begin] ?? NULL;
    if (!is_a($part, 'Drupal\paragraphs\ParagraphInterface')) {
      // The paragraph is not a paragraph entity, so we don't need to do anything.
      return;
    }
    if ($part->bundle() !== 'node_part_paragraph') {
      // The paragraph is not a node part paragraph, so we don't need to do
      return;
    }
    // Check if paragraph ues non-paragraph parts.
    $field_use_non_paragraph_parts = $part->get('field_use_non_paragraph_parts')
      ->getString();
    if ($field_use_non_paragraph_parts === '1') {
      // The paragraph uses non-paragraph parts, so we don't need to do
      return;
    }
    // Get the required fields.
    $required_fields = $this->requiredFields($field);
    // Prepare the build array remove unnecessary elements.
    $build = array_intersect_key(
      $build,
      array_flip(
        [
          ...array_filter(array_keys($build), fn($k) => str_starts_with($k, '#')),
          ...array_intersect(Element::children($build), $required_fields),
        ]
      )
    );
  }

  /**
   * Gets the paragraphs.
   *
   * @return array
   *   The paragraphs.
   */
  protected function getParagraphs(): array {
    $result = [
      'start' => NULL,
      'end' => NULL,
      'field' => NULL,
      'links' => [],
    ];

    $page_breaks_count = 0;

    $links = [];
    $field = NULL;
    foreach ($this->fields as $field) {
      /** @var \Drupal\paragraphs\ParagraphInterface[] $references */
      $references = $this->entity->get($field)->referencedEntities();
      $links = [];

      foreach ($references as $delta => $reference) {
        if ($reference->getEntityTypeId() !== 'paragraph') {
          continue;
        }

        switch ($reference->bundle()) {
          case 'node_part_link_paragraph':
            $links[] = $delta;
            break;

          case 'node_part_paragraph':
            $page_breaks_count++;

            if ($this->shouldSetAsEndParagraph($this->page, $page_breaks_count, $result['start'])) {
              $result['end'] = $delta;
              break 3;
            }

            if ($this->shouldSetAsStartParagraph($this->page, $page_breaks_count)) {
              $result['start'] = $delta;
            }
            break;
        }
      }

      if ($this->hasFoundParagraphs($result)) {
        break;
      }
    }

    $result['field'] = $field;
    $result['links'] = $links;

    return array_values($result);
  }

  /**
   * Filters the array by key range.
   *
   * @param array $array
   *   The array to filter.
   * @param int|null $min
   *   The minimum key.
   * @param int|null $max
   *   The maximum key.
   *
   * @return array
   *   The filtered array.
   */
  protected function filterByKeyRange(array $array, ?int $min, ?int $max): array {
    return array_filter(
      $array, fn($key) => ($min === NULL || $key > $min) &&
      ($max === NULL || $key <= $max),
      ARRAY_FILTER_USE_KEY
    );
  }

  /**
   * Determines if the paragraph should be set as the end paragraph.
   *
   * @param int $currentPage
   *   The current page number.
   * @param int $pageBreaksCount
   *   The number of page breaks.
   * @param int|null $startParagraph
   *   The start paragraph number.
   *
   * @return bool
   *   TRUE if the paragraph should be set as the end paragraph, FALSE
   *   otherwise.
   */
  protected function shouldSetAsEndParagraph(int $currentPage, int $pageBreaksCount, ?int $startParagraph): bool {
    return ($currentPage === 0) || ($startParagraph !== NULL && $currentPage <= $pageBreaksCount);
  }

  /**
   * Determines if the paragraph should be set as the start paragraph.
   *
   * @param int $currentPage
   *   The current page number.
   * @param int $pageBreaksCount
   *   The number of page breaks.
   *
   * @return bool
   *   TRUE if the paragraph should be set as the start paragraph, FALSE
   *   otherwise.
   */
  protected function shouldSetAsStartParagraph(int $currentPage, int $pageBreaksCount): bool {
    return $currentPage > 0 && $currentPage === $pageBreaksCount;
  }

  /**
   * Determines if the paragraphs have been found.
   *
   * @param array $result
   *   The result array.
   *
   * @return bool
   *   TRUE if the paragraphs have been found, FALSE otherwise.
   */
  protected function hasFoundParagraphs(array $result): bool {
    return $result['start'] !== NULL || $result['end'] !== NULL;
  }

  /**
   * Get the required fields.
   *
   * @param string $field
   *   The field name.
   *
   * @return array
   *   The required fields.
   */
  protected function requiredFields(string $field): array {
    // The field which using to render the paragraphs.
    $fields = [
      $field,
    ];
    // Let other modules to alter the fields.
    $event = new RequiredFieldsEvent($fields);
    $this->eventDispatcher->dispatch($event, RequiredFieldsEvent::class);

    // Get the fields from the event.
    return $event->getFields();
  }

}
